#-----------------------------------------------------------
# Testing for Trend in Benefit-Risk Analysis with Prioritized Multiple Outcomes
# Supplementary Material R code
# 08/15/2019
#
# Functions
# 1.DOOR algorithm for ranking 
# 2.Tukey's Trend test (Two tests based on permutation or based on asymptotic assumptions, adjusted alpha threshold)
# 3.Jonckheere-Terpstra test (Two tests based on permutation or based on asymptotic assumptions)
#-----------------------------------------------------------



# 1. DOOR algorithm for ranking (Based on the algorithm of Evans et al (2015))
#The higher DOOR.rank, the better clinical outcome the patient has (based on benefits and harms)
DOOR.rank <- function( df, #Dataset that contains all the variables that should be included for ranking and group label, class data.frame
                       var_order, #Desired order of variables in dataset df
                       ties.method = c("average","random") #By default, ties.method is average
) {
  library(dplyr)
  if (is.null(ties.method) == 1) {ties.method <- "average"}
  average <- function(x) mean(x)
  random <- function(x) sample(x, length(x))
  out <- df %>% mutate(r = do.call(order, c(df[,var_order])), rr=order(r)) %>% group_by_(.dots = var_order,add=T) %>% mutate(DOOR.rank = get(ties.method)(rr)) %>% as.data.frame() %>% select(-r, -rr) 
  rownames(out)=rownames(df)
  return(out)
}





#-----------------------------------------------------------





# 2.Tukey's Trend test (Two tests in function Tukey_trend_test() based on permutation or based on asymptotic assumptions)

Tukey_trend_test=function(rank, #The rank output from DOOR.rank function, make sure to have the actual dose levels for each subject, column name is "Dose", make sure it is numeric and controls have the lowest dose level
                    rsam = 1000, #Number of permutations to perform, default = 1000
                    alpha = 0.05 #Desired type I error level, defalt = 0.05
                    ){
  library(mvtnorm)
  n_sub = as.numeric(table(rank$Dose)) #Number of subjects for each group, ordered by group dose level from small to large
  n_group = length(n_sub) #Number of groups
  
  
  dose_level = sort(unique(rank$Dose))
  m=rep(0,n_group)
  for(i in 1:n_group){
    m[i]=mean(rank$DOOR.rank[which(rank$Dose == dose_level[i])]) 
  }
  
  
  #Ordinal
  dose_level_ordinal = 0:(n_group-1)
  x = sum(dose_level_ordinal * n_sub) / sum(n_sub)
  c1 = n_sub * ( dose_level_ordinal - x )
  test_tukey_ordinal = sum(c1 * m)
  

  #Arithmetic
  dose_level_arith = dose_level
  dose_level_arith[1] = 0
  x = sum(dose_level_arith * n_sub) / sum(n_sub)
  c2 = n_sub * (dose_level_arith - x)
  test_tukey_arith = sum(c2 * m)

  
  #Logarithmic
  dose_level_arithlog=c(log10(dose_level[2])-(log10(dose_level[3])-log10(dose_level[2]))*(dose_level[2]-dose_level[1])/(dose_level[3]-dose_level[2]),log10(dose_level[-1]))
  x = sum( dose_level_arithlog * n_sub) / sum(n_sub)
  c3 = n_sub * (dose_level_arithlog - x)
  test_tukey_arithlog = sum(c3 * m)
  
  
  
  #Permutation Tukey's test
  perm=list()
  rsam=10000
  for (z in 1:rsam){
    rank_perm=rank
    rank_perm$Dose_sam=sample(rank$Dose, replace = F)
    
    m_perm=rep(0,n_group)
    for(i in 1:n_group){
      m_perm[i]=mean(rank_perm$DOOR.rank[which(rank_perm$Dose_sam == dose_level[i])]) 
    }
    
  
    test_tukey_ordinal_perm = sum(c1 * m_perm)
    test_tukey_arith_perm = sum(c2 * m_perm)
    test_tukey_arithlog_perm = sum(c3 * m_perm)
    perm[[z]]=list(test_tukey_ordinal_perm=test_tukey_ordinal_perm,test_tukey_arith_perm=test_tukey_arith_perm,test_tukey_arithlog_perm=test_tukey_arithlog_perm)
  }
  test_ordinal_perm=unlist(lapply(perm,"[[",1))
  test_arith_perm=unlist(lapply(perm,"[[",2))
  test_arithlog_perm=unlist(lapply(perm,"[[",3))
  
  
  p_perm_ordinal=1/(rsam+1)*(sum(test_ordinal_perm >= test_tukey_ordinal)+1)
  p_perm_arith=1/(rsam+1)*(length(which(test_arith_perm >= test_tukey_arith))+1)
  p_perm_arithlog=1/(rsam+1)*(length(which(test_arithlog_perm >= test_tukey_arithlog))+1)
  
  #Final p-value for Tukey's permutation test
  p_tukey_perm=min(p_perm_arith,p_perm_ordinal,p_perm_arithlog)  
  
  
  #By asymptotic formula
  fit=lm(rank$DOOR.rank~rank$Dose)
  df = sum(n_sub)-n_group
  
  test_asymp_ordinal = sum(c1*m)^2 / (mean(fit$residuals^2) * sum(c1^2/n_sub))
  p_asymp_ordinal = 1-pf(test_asymp_ordinal,1,df)
  
  test_asymp_arithmetic = sum(c2*m)^2 / (mean(fit$residuals^2) * sum(c2^2/n_sub))
  p_asymp_arithmetic = 1-pf(test_asymp_arithmetic,1,df)
  
  test_asymp_arithlog = sum(c3*m)^2 / (mean(fit$residuals^2) * sum(c3^2/n_sub))
  p_asymp_arithlog = 1-pf(test_asymp_arithlog,1,df)
  
  #Final p-value for Tukey's test based on asymptotic formula
  tukey_p_asymp=min(p_asymp_ordinal,p_asymp_arithmetic,p_asymp_arithlog)
  
  
  #To adjust for tukey's trend test with the new alpha threshold at the desired type I error level, by default type I error level = 0.05
  cor12=cor(dose_level_ordinal,dose_level_arith)
  cor13=cor(dose_level_ordinal,dose_level_arithlog)
  cor23=cor(dose_level_arith,dose_level_arithlog)
  cormat1=matrix(c(1,cor12,cor13,cor12,1,cor23,cor13,cor23,1),3,3)
  #If desired type 1 error if set to 0.05
  t=as.numeric(qmvt(1-alpha,tail="both.tails",maxiter=1000,corr=cormat1,interval=c(0,1),df=df)[1])
  alpha_adjusted=pt(t,df=df,lower.tail = FALSE)*2 
  
  
  
  results = list(tukey_perm_statistics = perm, p_tukey_perm = p_tukey_perm, tukey_asymp_statistics = c(test_asymp_ordinal = test_asymp_ordinal,test_asymp_arithmetic = test_asymp_arithmetic,test_asymp_arithlog = test_asymp_arithlog ),tukey_p_asymp = tukey_p_asymp, alpha_adjusted = alpha_adjusted)
  return(results)

  
}





#-----------------------------------------------------------





# 3.Jonckheere-Terpstra test (Two tests based on permutation or based on asymptotic assumptions)


JT_test=function(rank, #The rank output from DOOR.rank function
                 rsam = 1000 #Number of permutations to perform, default = 1000
                 ){
  
  
  n_sub = as.numeric(table(rank$Dose)) #Number of subjects for each group, ordered by group dose level from small to large
  n_group = length(n_sub) #Number of groups
  dose_level = sort(unique(rank$Dose))
 
  
 #Construct the u statistics
  ustat=function(x,y){
    u=matrix(nrow=dim(x)[1],ncol=dim(y)[1])
    for (i in 1:dim(x)[1]){
      for (j in 1:dim(y)[1]){
        
        if(x$DOOR.rank[i]<y$DOOR.rank[j]) {
          u[i,j]=1
        } else if(x$DOOR.rank[i]==y$DOOR.rank[j]) {
          u[i,j]=0.5
        } else {u[i,j]=0}
        
      }
    }
    return(u)
  }


    #By aymptotic formula
 
    u_stat = 0
  
    for(j in 1:(n_group-1)){
      for(z in 1:(n_group-j)){
        u_stat = c(u_stat,sum(ustat(rank[which(rank$Dose == dose_level[j]),],rank[which(rank$Dose == dose_level[j+z]),])))
      }
    }
    
    u_stat = u_stat[-1]
    u_all = sum(u_stat)
    N = sum(n_sub)
    
    expectu = (N^2-sum(n_sub^2))/4
    
    tie = as.numeric(table(rank$DOOR.rank))
    
    varu = ((N*(N-1)*(2*N+5)-sum(n_sub*(n_sub-1)*(2*n_sub+5)) - sum(tie*(tie-1)*(2*tie+5)))/72)+
      (sum(n_sub*(n_sub-1)*(n_sub-2))+sum(tie*(tie-1)*(tie-2)))/(36*N*(N-1)*(N-2))+
      (sum(n_sub*(n_sub-1))+sum(tie*(tie-1)))/(8*N*(N-1))
    
    z_u = (u_all-expectu) / sqrt(varu)
    p_jon_asymp = pnorm(z_u,lower.tail=F)
  

    #Permutation Jonckheere test
    library(clinfun)
    joncktest_perm = jonckheere.test(x=rank$DOOR.rank,g=rank$Dose,alternative = "increasing",nperm = rsam)
    test_stat_jon_perm = joncktest_perm$statistic
    p_jon_perm = joncktest_perm$p.value
    
    results = list(JT_score_asymp = u_all, test_stat_jon_asymp = z_u, variance_jon_asymp = varu, p_jon_asymp = p_jon_asymp, JT_score_perm = test_stat_jon_perm, p_jon_perm = p_jon_perm, jon_perm_all = joncktest_perm)
    return(results)
}





#-----------------------------------------------------------
